/* Seven2one Corporate Design Colors */

:root {
  /* Primary Colors */
  --s2o-orange: #EF8200;
  --s2o-green: #10A384;
  --s2o-dark-gray: #707070;
  --s2o-white: #FFFFFF;

  /* Secondary Colors */
  --s2o-blue: #016CA7;
  --s2o-medium-gray: #A5A5A5;
  --s2o-light-gray: #DDDDDD;
  --s2o-white-smoke: #F2F2F2;

  /* Text Colors */
  --s2o-black: #000000;

  /* 30% Opacity Colors */
  --s2o-orange-30: #FADAB4;
  --s2o-green-30: #B7E3DA;
  --s2o-blue-30: #B2D3E4;
}

/* Light Mode - Seven2one Orange & Green */
[data-md-color-scheme="default"] {
  --md-primary-fg-color: var(--s2o-orange);
  --md-primary-fg-color--light: var(--s2o-orange-30);
  --md-primary-fg-color--dark: #D67300;
  --md-primary-bg-color: var(--s2o-white);
  --md-primary-bg-color--light: var(--s2o-white-smoke);

  --md-accent-fg-color: var(--s2o-green);
  --md-accent-fg-color--transparent: rgba(16, 163, 132, 0.1);
  --md-accent-bg-color: var(--s2o-white);
  --md-accent-bg-color--light: var(--s2o-green-30);

  /* Typography */
  --md-text-color: var(--s2o-black);
  --md-text-link-color: var(--s2o-blue);

  /* Code blocks */
  --md-code-bg-color: var(--s2o-white-smoke);
  --md-code-fg-color: var(--s2o-dark-gray);
}

/* Dark Mode - Seven2one Green primary */
[data-md-color-scheme="slate"] {
  --md-primary-fg-color: var(--s2o-green);
  --md-primary-fg-color--light: var(--s2o-green-30);
  --md-primary-fg-color--dark: #0D8268;

  --md-accent-fg-color: var(--s2o-orange);
  --md-accent-fg-color--transparent: rgba(239, 130, 0, 0.1);

  /* Text */
  --md-typeset-a-color: var(--s2o-orange);

  /* Code blocks - Override Material's default variables */
  --md-code-bg-color: #1e1e1e;
  --md-code-fg-color: #e0e0e0;
}

/* Headers - Ubuntu Font */
.md-typeset h1,
.md-typeset h2,
.md-typeset h3,
.md-typeset h4,
.md-typeset h5,
.md-typeset h6 {
  font-family: 'Ubuntu', sans-serif;
  font-weight: 400;
  color: var(--s2o-dark-gray);
}

[data-md-color-scheme="slate"] .md-typeset h1,
[data-md-color-scheme="slate"] .md-typeset h2,
[data-md-color-scheme="slate"] .md-typeset h3,
[data-md-color-scheme="slate"] .md-typeset h4 {
  color: var(--s2o-white);
}

/* Body Text - Ubuntu */
.md-typeset {
  font-family: 'Ubuntu', sans-serif;
  font-weight: 400;
}

/* Bold Text - Ubuntu Medium */
.md-typeset strong,
.md-typeset b {
  font-weight: 500;
}

/* Navigation - Orange highlights */
.md-nav__link--active {
  color: var(--s2o-orange);
}

.md-nav__link:hover {
  color: var(--s2o-green);
}

/* Buttons */
.md-button {
  background-color: var(--s2o-orange);
  color: var(--s2o-white);
  border-color: var(--s2o-orange);
}

.md-button:hover,
.md-button:focus {
  background-color: var(--s2o-green);
  border-color: var(--s2o-green);
}

.md-button--primary {
  background-color: var(--s2o-green);
  border-color: var(--s2o-green);
}

.md-button--primary:hover,
.md-button--primary:focus {
  background-color: var(--s2o-orange);
  border-color: var(--s2o-orange);
}

/* Code blocks - WhiteSmoke background (Light Mode) */
[data-md-color-scheme="default"] .md-typeset code {
  background-color: var(--s2o-white-smoke) !important;
  color: var(--s2o-dark-gray) !important;
}

/* Code blocks - Dark background with light text (Dark Mode) */
/* Target all possible code elements */
[data-md-color-scheme="slate"] .md-typeset code,
[data-md-color-scheme="slate"] .md-typeset .highlight code,
[data-md-color-scheme="slate"] .md-typeset div.highlight code {
  background-color: #1e1e1e !important;
  color: #f2f2f2 !important;
}

/* Inline code - better contrast in dark mode */
[data-md-color-scheme="slate"] .md-typeset p code,
[data-md-color-scheme="slate"] .md-typeset li code,
[data-md-color-scheme="slate"] .md-typeset td code,
[data-md-color-scheme="slate"] .md-typeset th code {
  background-color: #2d2d2d !important;
  color: var(--s2o-orange) !important;
  border: 1px solid rgba(239, 130, 0, 0.3) !important;
  padding: 0.2em 0.4em !important;
  border-radius: 3px !important;
}

/* Admonitions - Seven2one colors */
.md-typeset .admonition.note,
.md-typeset details.note {
  border-color: var(--s2o-blue);
}

.md-typeset .note > .admonition-title,
.md-typeset .note > summary {
  background-color: rgba(1, 108, 167, 0.1);
}

.md-typeset .admonition.warning,
.md-typeset details.warning {
  border-color: var(--s2o-orange);
}

.md-typeset .warning > .admonition-title,
.md-typeset .warning > summary {
  background-color: var(--s2o-orange-30);
}

.md-typeset .admonition.tip,
.md-typeset details.tip {
  border-color: var(--s2o-green);
}

.md-typeset .tip > .admonition-title,
.md-typeset .tip > summary {
  background-color: var(--s2o-green-30);
}

/* Search - Light Mode */
[data-md-color-scheme="default"] .md-search__input {
  background-color: var(--s2o-white-smoke);
  color: var(--s2o-black);
}

[data-md-color-scheme="default"] .md-search__input::placeholder {
  color: var(--s2o-medium-gray);
}

/* Search - Dark Mode */
[data-md-color-scheme="slate"] .md-search__input {
  background-color: #2d2d2d;
  color: #f2f2f2;
}

[data-md-color-scheme="slate"] .md-search__input::placeholder {
  color: #8a8a8a;
}

/* Search results - Dark Mode */
[data-md-color-scheme="slate"] .md-search-result__meta {
  color: #8a8a8a;
}

[data-md-color-scheme="slate"] .md-search-result__article {
  background-color: #2d2d2d;
}

[data-md-color-scheme="slate"] .md-search-result__title {
  color: #f2f2f2;
}

[data-md-color-scheme="slate"] .md-search-result__teaser {
  color: #b0b0b0;
}

/* Links */
.md-typeset a {
  color: var(--s2o-blue);
}

.md-typeset a:hover,
.md-typeset a:focus {
  color: var(--s2o-green);
}

/* Tabs */
.md-tabs {
  background-color: var(--s2o-orange);
}

.md-tabs__link--active {
  color: var(--s2o-white);
}

/* Header */
.md-header {
  background-color: var(--s2o-orange);
}

[data-md-color-scheme="slate"] .md-header {
  background-color: var(--s2o-dark-gray);
}

/* Sidebar */
.md-sidebar--primary {
  background-color: var(--s2o-white-smoke);
}

[data-md-color-scheme="slate"] .md-sidebar--primary {
  background-color: #2d2d2d;
}

/* Code highlighting - Light Mode */
[data-md-color-scheme="default"] .highlight .hll { background-color: var(--s2o-orange-30); }
[data-md-color-scheme="default"] .highlight .c { color: var(--s2o-medium-gray); } /* Comment */
[data-md-color-scheme="default"] .highlight .k { color: var(--s2o-blue); font-weight: 500; } /* Keyword */
[data-md-color-scheme="default"] .highlight .s { color: var(--s2o-green); } /* String */
[data-md-color-scheme="default"] .highlight .nf { color: var(--s2o-orange); } /* Function name */

/* Code highlighting - Dark Mode - High Contrast */
[data-md-color-scheme="slate"] .highlight .hll { background-color: rgba(239, 130, 0, 0.3); }
[data-md-color-scheme="slate"] .highlight .c { color: #8a8a8a; } /* Comment - lighter gray */
[data-md-color-scheme="slate"] .highlight .k { color: #5cb3ff; font-weight: 500; } /* Keyword - bright blue */
[data-md-color-scheme="slate"] .highlight .s { color: #4cd964; } /* String - bright green */
[data-md-color-scheme="slate"] .highlight .nf { color: #ffa726; } /* Function name - bright orange */
[data-md-color-scheme="slate"] .highlight .nb { color: #f48fb1; } /* Built-in - pink */
[data-md-color-scheme="slate"] .highlight .nc { color: #ffeb3b; } /* Class name - yellow */
[data-md-color-scheme="slate"] .highlight .nn { color: #80deea; } /* Module name - cyan */
[data-md-color-scheme="slate"] .highlight .mi,
[data-md-color-scheme="slate"] .highlight .mf { color: #ce93d8; } /* Numbers - purple */

/* Code blocks background - Dark Mode */
/* Override ALL Material Theme code block backgrounds */
[data-md-color-scheme="slate"] .md-typeset pre,
[data-md-color-scheme="slate"] .md-typeset pre > code,
[data-md-color-scheme="slate"] .md-typeset pre code,
[data-md-color-scheme="slate"] .md-typeset .codehilite,
[data-md-color-scheme="slate"] .md-typeset .codehilite pre {
  background-color: #1e1e1e !important;
  color: #e0e0e0 !important;
}

[data-md-color-scheme="slate"] .md-typeset .highlight,
[data-md-color-scheme="slate"] .md-typeset .highlight pre,
[data-md-color-scheme="slate"] .md-typeset div.highlight,
[data-md-color-scheme="slate"] .md-typeset div.highlight pre {
  background-color: #1e1e1e !important;
}

[data-md-color-scheme="slate"] .md-typeset .highlight code,
[data-md-color-scheme="slate"] .md-typeset div.highlight code {
  background-color: transparent !important;
}

/* General text in code blocks - Dark Mode */
[data-md-color-scheme="slate"] .highlight,
[data-md-color-scheme="slate"] .highlight .n,
[data-md-color-scheme="slate"] .highlight .o {
  color: #e0e0e0 !important;
}

/* Footer */
.md-footer {
  background-color: var(--s2o-dark-gray);
}

.md-footer-meta {
  background-color: rgba(112, 112, 112, 0.9);
}
